function out_gamma=MCLSTV(B,par,opts)
%--------------------------------------------------------------------------
%  MCLSTV Algorithm. Ref. 
%    Iterative Constrained Minimization for Vectorial TV Image Deblurring
%     K. Chen, E.Loli Piccolomini,   F. Zama
%     Journal of Mathematical Imaging and Vision, 2015
%      http://dx.doi.org/10.1007/s10851-015-0599-3 
% 
%--------------------------------------------------------------------------
%
%  input: 
%      B : blurred noisy image
%      par, opts : parameters defined in setpar.m 
%  output: 
%      out_gamma : output structure with fiels:
%            out_gamma.sol : solution image
%            out_gamma.itadapt : number of adaptation iterations
%            out_gamma.IT_out : outer iterations (k in Algorithm 3)
%            out_gamma.IT_in  :  inner iteration (FTVd method)
%            out_gamma.lambda : Multiplier value (\lambda)
%            out_gamma.gamma : last value gamma
%            out_gamma.time : total comutation time (sec.)
%--------------------------------------------------------------------------
H=par.H;
LAMBDA=par.LAMBDA;
Norma='L2';
off=par.off;
params=par.params;
X_true=par.sol_true;
IBig=X_true;
TOL_ratio=par.TOL_ratio;
tol=params(1); 
maxit=params(2); 
ks=params(3); 
lambda=params(4); 
verbose=params(5);
gamma_par=params(6); 
adapt=params(7);
beta=params(8);
maxit_adapt=params(9);
N1=2;

%
%----------------------------
[NB,MB,K]=size(B);
[N,M,K]=size(X_true);
Scala=1;
%
Noisy_imm=B(off+1:size(B,1)-off,off+1:size(B,2)-off,:);
     Res=0;
     H1=B-HF(LAMBDA,IBig);
     Res=dot(H1(:),H1(:)); 
     f=MTV(IBig,beta);
     gamma_true=f;
     RR1=Res_fun_3(IBig,B,LAMBDA);
    Res_true=norm(RR1(:));
%
% Lambda Starting value
%
Time_total=0;
Bv=fft2(B);
%
%
    IT_in=0;
fprintf('+');
     if lambda == -1,
         Sa=abs((LAMBDA{1,1}+LAMBDA{2,2}+LAMBDA{3,3}))/3;
         [so,ind]=sort(Sa,'descend');Bs=Bv(ind);
         BB1=abs(Bs(1:M:end));
         SS1=so(1:M:end);V2=BB1./SS1;
         [ZZ,indZ]=min(V2);
         lambda=SS1(indZ);
	     out = FTVd_v4(H,B,1/lambda,Norma,opts);
    	Time_total=Time_total+out.time;
    	X=out.sol;IT_in=out.itr;IT_out=0;
     end
    X=X.*(X>0);
    f=MTV(X,beta);
    X0=X(off+1:size(B,1)-off,off+1:size(B,2)-off,:);

    gam_L=f;
    
    RR_1=Res_fun_3(X,B,LAMBDA); 
    res0=norm(RR_1(:),N1);
    SNR0=snr(X0,X_true);
    if verbose,
      figure; imshow(X0);title(['gamma 0 =', ...
      num2str(gam_L,'%0.4e'),'SNR=',num2str(SNR0,'%0.4e')]);
    end
    %fprintf('--------------------------------------------------------------\n');
     NT=norm(X_true(:));
    it_gamma=0;
    if gamma_par == -1
        gamma=gam_L;
    else
        gamma=gamma_par;it_gamma=it_gamma+1;
        [out_solve]=solve_for_gamma(gamma,lambda,H, B, IBig, ...
             LAMBDA,maxit,beta,tol,ks,verbose,Norma,off,X_true,opts);
        Time_total=Time_total+out_solve.time;
        IT_in=IT_in+out_solve.itint;
        IT_out=IT_out+out_solve.itout;
        RR_0=Res_fun_3(out_solve.sol,B,LAMBDA); 
        res0=norm(RR_0(:),N1);
        
    end
    gam_L=gamma;
   cond_exit=0; 
   %    
    continua=1;it_gamma=it_gamma+1;
    percP=0.001;
    gamma_up=gam_L*(1+percP);
    Type_step='Grow';
    %
    [out_solve]=solve_for_gamma(gamma_up,lambda,H, B, IBig, ...
             LAMBDA,maxit,beta,tol,ks,verbose,Norma,off,X_true,opts);
    it_gamma=it_gamma+1;
    Time_total=Time_total+out_solve.tempo;
    IT_in=IT_in+out_solve.itint;
    IT_out=IT_out+out_solve.itout;
    RR_up=Res_fun_3(out_solve.sol,B,LAMBDA); 
    res_up=norm(RR_up(:),N1);
    P=(res_up-res0)/(gamma_up-gam_L);
    if P > 0, 
      Type_step='Decr';
      gamma_old=gam_L;res_old=res0; 
     else
       gamma_old=gamma_up; res_old=res_up;    
    end
%--------------------------------------------------------------------------
%  Adaptation
%--------------------------------------------------------------------------
    percP=par.percP;
    if strcmp(Type_step,'Grow'),
        gamma_new=gamma_old*(1+percP);
     elseif strcmp(Type_step,'Decr'),
        gamma_new=gamma_old*(1-percP);
    end
    while continua
    fprintf('+');
%--------------------------------------------------------------------------
      it_gamma=it_gamma+1;
      [out_solve]=solve_for_gamma(gamma_new,lambda,H, B, IBig, ...
             LAMBDA,maxit,beta,tol,ks,verbose,Norma,off,X_true,opts);
        Time_total=Time_total+out_solve.tempo; 
        IT_in=IT_in+out_solve.itint;
        IT_out=IT_out+out_solve.itout;
      SNRnew=out_solve.SNR;
      X=out_solve.sol;X=X.*(X>0);
      RR=Res_fun_3(X,B,LAMBDA); 
      res_new=norm(RR(:),N1);
      Xnew=X(off+1:size(B,1)-off,off+1:size(B,2)-off,:);    
        if verbose
            figure; imshow(Xnew);title(['gamma new=', ...
           num2str(gamma,'%0.4e'),'SNR=',num2str(SNRnew,'%0.4e')]);
        end      
       P_new=(res_new-res_old)/(gamma_new-gamma_old);
       
       if strcmp(Type_step,'Grow'),
%----------------------------------------------------------------------------
%   Type_step = grow
%----------------------------------------------------------------------------
         if P_new > TOL_ratio
             percP=max(percP/10,1.e-4);
             gamma_new=gamma_old*(1+percP);
          elseif P_new < -TOL_ratio
             gamma_old=gamma_new;
             res_old=res_new;
             gamma_new=gamma_old*(1+percP);

           else
            cond_exit=1;
          end
       elseif strcmp(Type_step,'Decr'),
          if P_new > TOL_ratio          
             percP=max(percP/10,1.e-4);
             gamma_new=gamma_old*(1-percP);
          elseif P_new < -TOL_ratio
             gamma_old=gamma_new;
             res_old=res_new;
             gamma_new=gamma_old*(1-percP);
          else
            cond_exit=1;
          end
       end
%----------------------------------------------------------------------------
%  exit test
%----------------------------------------------------------------------------
      if cond_exit,
          continua =0;
      else        
          continua = it_gamma < maxit_adapt;
          if continua==0, fprintf(' Uscita x iter');end
           if verbose
              fprintf(' iter adapt =%d gamma=%e SNR=%e \n',it_gamma, gamma_new, SNRnew );
          end
      end
    end    
IT_out=IT_out+out_solve.itout;
IT_in=IT_in+out_solve.itint;

out_gamma.sol=out_solve.sol;
out_gamma.itadapt=it_gamma;
out_gamma.IT_out=IT_out;
out_gamma.IT_in = IT_in;
out_gamma.lambda=out_solve.lambda;
out_gamma.gamma=out_solve.gamma;
out_gamma.time=Time_total;
%=========================================================================
%-------------------------------------------------------------------------
function [out_par]=solve_for_gamma...
    (gamma,lambda,H, B, IBig, LAMBDA,maxit,beta,tol,ks,verbose,Norma,off,I,opts)
% 
%  solve the minimization problem 
%
%     min || H x - y || s.t. MTV(x) < gamma
%
%--------------------------------------------------------------------------
if tol==-1, tol=1.e-3;end % 
if maxit==-1, maxit=10;end %
if ks==-1, ks = 2;end;%
%parameters setting
tau_r=tol;
tau_a=tol;
%---------------------------------------------------------------------------------------------------------------
%
% Computation of the first value lambda_0 s.t. MTV(lambda_0) < gamma
% 
    lambda_start = lambda;
    Time_total=0;IT_in=0;
    out = FTVd_v4(H,B,1/lambda_start,Norma,opts);
    Time_total=Time_total+out.time;
    X=out.sol;f=MTV(X,beta);gam_start=f;
    Err=norm(X(:) - IBig(:))/norm(IBig(:));
    IT_in=IT_in+out.itr;
    F_start=gam_start-gamma;
    [out_interval]=find_interval_adapt(lambda_start, F_start, Err, gamma, H, B, IBig, LAMBDA,maxit,beta,Norma);
     GG=out_interval.GG;Lambda=out_interval.Lambda;
     lambda0=out_interval.lambda0;
%--------------------------------------------------------------------------
F0=out_interval.F0;
lambda_1=out_interval.lambda_1;
F1=out_interval.F1;
it_init=out_interval.it_init;
IT0_in=out_interval.IT_in;
Time_total=Time_total+out_interval.Time_total;
FL=out_interval.FL;
IT_in=IT_in+IT0_in;
%--------------------------------------------------------------------------
it_uscita =0; it=0;
if FL
%
%  The function find_interval_adapt could not find a suitable value lambda_0 s.t.  MTV(lambda_0) < gamma 
%  
% The computed solution X is relative to  the last computed value lambda_0 s.t. MTV(lambda_0) > gamma.
%  
     lambda_new=lambda0;
    out = FTVd_v4(H,B,1/lambda_new,Norma,opts);
    IT_in=IT_in+out.itr;Time_total=Time_total+out.time;
    X=out.sol;f=MTV(X,beta);gam_new=f; 
    X_new=X(off+1:size(B,1)-off,off+1:size(B,2)-off,:);
   
    Time_total=Time_total+out.time;
    out_par.err=FL;verbose=0;
else
%
%  computation of the (lambda_k, X_k) succession
%
   out_par.err=0;
   continua =1;
   Val_G=tau_r*abs(F1)+ tau_a;
   it_fl=1;
    ks=3;
%
    X_old=X(off+1:size(B,1)-off,off+1:size(B,2)-off,:);
%
    cond_diff=0; Dist_abs_old=[];
    while it < maxit
           it=it+1;
           if it >ks 
             lambda_new=lambda_1 - F1*(lambda_1 - lambda0)/(F1-F0);
           else
             lambda_new=(lambda_1+lambda0)*0.5;
           end
           
	       out = FTVd_v4(H,B,1/lambda_new,Norma,opts);
           IT_in=IT_in+out.itr;Time_total=Time_total+out.time;
	       X=out.sol;f=MTV(X,beta);gam_new=f; 
           X_new=X(off+1:size(B,1)-off,off+1:size(B,2)-off,:);
           Dist_abs=norm(X_new(:)-X_old(:));
           if ne(isempty(Dist_abs_old),1),
               Diff_imm=abs(Dist_abs-Dist_abs_old);
               cond_diff= Diff_imm < 5.e-3;
           end
	       Fnew=gam_new-gamma;
           GG(2+it)=Fnew; 
           Lambda(2+it)=lambda_new;
           EE(2+it)=norm(X(:) - IBig(:))/norm(IBig(:));
	    if ((abs(Fnew) <= Val_G) && it_fl ) || (cond_diff && it_fl )
               it_uscita = it+2;it_fl=0;
               I_uscita = X(off+1:size(B,1)-off,off+1:size(B,2)-off,:);             
                out_par.sol=X;
               out_par.lambda=lambda_new;
               out_par.gamma=gam_new;
               out_par.itout=it+it_init;
               out_par.itint=IT_in;
               out_par.tempo=Time_total;
                SNR0=snr(I_uscita,I);
                out_par.SNR=SNR0;
                out_par.tempo=Time_total;
            end  
           if F0*Fnew < 0
             lambda_1=lambda_new;F1=Fnew;
           else   
             lambda0=lambda_new;F0=Fnew;
           end
           Dist_abs_old=Dist_abs;
        
    end
end
if it_uscita == 0
    it_uscita=it;
    I_uscita=X(off+1:size(B,1)-off,off+1:size(B,2)-off,:);
     out_par.sol=X;
    out_par.lambda=lambda_new;
    out_par.gamma=gam_new;
    out_par.itout=it+it_init;
    out_par.itint=IT_in;
    out_par.tempo=Time_total;
    SNR0=snr(I_uscita,I);
    out_par.SNR=SNR0;
end

if verbose,
    [EE_best, it_best]=min(EE);
    figure;plot(1:length(GG),GG+gamma,'.-',1:length(GG),ones(size(GG))*gamma,'-r',it_uscita,GG(it_uscita)+gamma,'.g');
    legend(' MTV(x_k)', '\gamma', 'exit' );xlabel('k')
    figure;plot(1:it_uscita,GG(1:it_uscita)+gamma,'.-',1:it_uscita,ones(it_uscita,1)*gamma,'-r');
    legend(' MTV(x_k)', '\gamma');xlabel('k')
    figure; plot(1:length(EE),EE,'.-b', it_uscita,EE(it_uscita+2),'og',it_best,EE_best,'*r') ;    
end
%==========================================================================
%
function [out]=find_interval_adapt(lambda_start, F_start,Err_start, gamma, H, B,IBig,LAMBDA,maxit,beta,Norma)
%
%   find a suitable interval [lambda0, lambda1] s.t. 
%     MTV(x(lambda0))>gamma  && MTV(x(lambda1))<gamma
%
IT_in=0;
Time_total=0;
it_lambda0=0;
it_lambda_1=0;
lambda_init=lambda_start;
lambda_old=lambda_start;F_old=F_start;Err_old=Err_start;
%
MaxC=1;FL=0;
cont_ext=1;st_count=0;
GG=[];
Lambda=[];lambda0=[];F0=[];
while cont_ext
if F_start > 0 
    continua = 1;
    lambda_new=lambda_start;
    while continua
       it_lambda0=it_lambda0+1;
       lambda_new = lambda_new*5;Lam(it_lambda0)=lambda_new;
%
       out = FTVd_v4(H,B,1/lambda_new,Norma);
       Time_total=Time_total+out.time;
       X=out.sol;f=MTV(X,beta);
       gam_new=f;
       Err_new=norm(X(:) - IBig(:))/norm(IBig(:));
       IT_in=IT_in+out.itr;
       F_new=gam_new-gamma;
       continua = F_new > 0  && it_lambda0 < maxit ;
       if continua
           lambda_old=lambda_new;gam_old = gam_new;
           F_old=F_new;Err_old=Err_new;
       end
    end
    if it_lambda0 < maxit
        F2=F_new;lambda_1=lambda_new;F1=F2;
        GG(2)=F2; Lambda(2)=lambda_1; 
        GG(1)=F_old;Lambda(1)=lambda_old;
        lambda0 = lambda_old; F0=F_old;
    else
       fprintf(' it_lambda0=%d lambda=%0.5e  F=%0.5e \n',...
            it_lambda0,lambda_new,F_new); 
        lambda_start=lambda_init/5;
         fprintf('- Suitable value lambda1= not found, restarted with lambda=%e \n',lambda_start);
         st_count=st_count+1;        FL=1;
    end                    
else
%
% find lambda0 s.t. -R(lambda0) -gamma <0
%
    it_lambda_1=0;continua=1;
    lambda_new=lambda_start;
      while continua
       it_lambda_1=it_lambda_1+1;
       lambda_new = lambda_new/5;Lam(it_lambda_1)=lambda_new;
%
       out = FTVd_v4(H,B,1/lambda_new,Norma);
       Time_total=Time_total+out.time;
       X=out.sol;f=MTV(X,beta);gam_new=f;
       Err_new=norm(X(:) - IBig(:))/norm(IBig(:));
       IT_in=IT_in+out.itr;
       %RR_0=Res_fun_3(X,B,LAMBDA);
       F_new=gam_new-gamma;
       continua = F_new < 0 && it_lambda_1 < maxit ;
       if continua
           lambda_old=lambda_new;gam_old = gam_new;
           F_old=F_new;Err_old=Err_new;
       end
      end
    if it_lambda_1 < maxit
        lambda0 = lambda_new;F0=F_new;
        GG(1)=F0; Lambda(1)=lambda0;%EE(1)=Err_new;
        F1=F_old;lambda_1=lambda_old;Err_1=Err_old;
        GG(2)=F1;Lambda(2)=lambda_1;%EE(2)=Err_1;
    else
            fprintf(' it_lambda1=%d lambda=%0.5e  F=%0.5e \n',...
            it_lambda_1,lambda_new,F_new); 
  
         lambda_start=lambda_init*5;
         fprintf('+ Suitable value lambda1 not found restarted with lambda=%e \n',lambda_start);
         st_count=st_count+1;
        FL=1;
       
    end
end
cont_ext = (st_count < MaxC) && FL;
if cont_ext,
    FL=0; 
    lambda_init=lambda_start;
    out = FTVd_v4(H,B,1/lambda_init,Norma);
    X=out.sol;f=MTV(X,beta);gam_new=f;
    F_start=gam_new-gamma;F_old=F_start;
end
end
it_init=it_lambda0+it_lambda_1;
if FL
    fprintf(' Suitable value lambda1 not found lambda=%e \n',lambda_new);
    lambda0=Lambda(2);F0=F_start;lambda_1=lambda_new;F1=F_new;
end
out.GG=GG;
out.Lambda=Lambda;
out.lambda0=lambda0;
out.F0=F0;
out.lambda_1=lambda_1;
out.F1=F1;
out.it_init=it_init;
out.IT_in=IT_in;
out.Time_total=Time_total;
out.FL=FL;
%=========================================================================
function Bbig=Res_fun_3(X,b,LAMBDA)
    %
    % Bbig= H'(b-Hx)
    % 
    HX=HF(LAMBDA,X);
    b=b-HX;
    Fb2=fft2(b);
    for i=1:3
        Bbig(:,:,i)=(conj(LAMBDA{i,1}).*Fb2(:,:,1) +conj(LAMBDA{i,2}).*Fb2(:,:,2) + conj(LAMBDA{i,3}).*Fb2(:,:,3));
    end
    Bbig=real(ifft2(Bbig));
%===========================================================================
function Mtv=MTV(U,beta)
V=zeros(size(U));
%
[D1,D2] = ForwardD3(U);
 Gi=zeros(size(U,1),size(U,2));
 for k=1:3
    Gi=Gi+D1(:,:,k).^2+D2(:,:,k).^2;%%%%%
end
%
Mtv=sum(sqrt(Gi(:)+beta^2));

function [Dux,Duy] = ForwardD3(U)
  Dux = [diff(U,1,2), U(:,1,:) - U(:,end,:)];
  Duy = [diff(U,1,1); U(1,:,:) - U(end,:,:)];
%========================================================================
function Bbig=HF(LAMBDA,IIbig)
F2_I=fft2(IIbig);
for i=1:3
   Bbig(:,:,i)=LAMBDA{i,1}.*F2_I(:,:,1)+ LAMBDA{i,2}.*F2_I(:,:,2)+LAMBDA{i,3}.*F2_I(:,:,3);
end
Bbig=real(ifft2(Bbig));
%==========================================================================
