%--------------------------------------------------------------------------
%   Script to test  MCLSTV algorithm in paper:
%
%    Iterative Constrained Minimization for Vectorial TV Image Deblurring
%     K. Chen, E.Loli Piccolomini,   F. Zama
%     Journal of Mathematical Imaging and Vision, 2015
%      http://dx.doi.org/10.1007/s10851-015-0599-3 
%--------------------------------------------------------------------------
clear all
close all
clc
addpath FTVD4
%--------------------------------------------------------------------------
% p codes of FTVd 4 are included. Matlab codes available at
%   http://www.caam.rice.edu/~optimization/L1/ftvd/
%--------------------------------------------------------------------------
% Gauss noise 
%--------------------------------------------------------------------------
noise=0.05; % noise level
nome='Npeppers2.png'; % True image file
%Blur_Kernels='1K';% kernel H1, eqn. (24) in article. 
Blur_Kernels='9K';% kernel H2, eqn. (25) in article. 
%--------------------------------------------------------------------------
randn('seed',0);rand('seed',0);
[I,Bn,par] = Gen_Blur_Noisy_Im(nome,noise,Blur_Kernels);
%--------------------------------------------------------------------------
snrBn = snr(Bn,I);
Erel0 = norm(Bn(:)-I(:))/norm(I(:));
MSE0=norm(Bn(:)-I(:))/numel(I);
%--------------------------------------------------------------------------
fprintf('----------  Data ------------------------------------------\n')
fprintf('\n Test image: %s  \n Blur Kernels %s \n',nome,Blur_Kernels);
fprintf(' Blur Noisy Relative Error=%e \n Blur Noisy  SNR =%7.3f \n',Erel0,snrBn);
fprintf(' Blur Noisy Mean Square Error = %e \n',MSE0)
fprintf(' Noise level %e \n \n',noise);
%--------------------------------------------------------------------------
[par,opts]=setpar(par);
out=MCLSTV(Bn,par,opts);
%--------------------------------------------------------------------------
Im_Rest=out.sol;
SNR_Rest=snr(Im_Rest,I);
Erel_Rest = norm(Im_Rest(:)-I(:))/norm(I(:));
MSE_Rest=norm(Im_Rest(:)-I(:))/numel(I);
fprintf('\n---------- Results -----------------------------------------------\n')
fprintf(' Restored Relative Error                              : %e \n',Erel_Rest);
fprintf(' Restored SNR                                         : %7.3f \n',SNR_Rest);
fprintf(' Restored Mean Square Error                           : %e \n',MSE_Rest);
fprintf(' gamma      : %0.5e          lambda             : %0.5e \n',out.gamma, out.lambda);
fprintf(' outer iterations (k)   : %d  inner iteration (it)    : %d \n',out.IT_out, out.IT_in);
fprintf(' gamma adaptation steps                               : %d \n',out.itadapt);
fprintf(' sec.                                                 : %d \n',out.time);
fprintf('------------------------------------------------------------------\n')
figure; imshow(I); title('True Image');
figure;imshow(Bn);title('Blurred Noisy image')
figure;imshow(Im_Rest);title('Restored Image');
